%
% FMMA CRC Book/JRR/JS/ExampSolSW\Example 8.7ANFIS

function [sys,x0,str,ts] = sfunEKFDmod(t,x,u,flag,X0,H,G)
%SFUNTMPL General M-file S-function template
%   With M-file S-functions, you can define you own ordinary differential
%   equations (ODEs), discrete system equations, and/or just about
%   any type of algorithm to be used within a Simulink block diagram.
%
%   The general form of an M-File S-function syntax is:
%       [SYS,X0,STR,TS] = SFUNC(T,X,U,FLAG,P1,...,Pn)
%
%   What is returned by SFUNC at a given point in time, T, depends on the
%   value of the FLAG, the current state vector, X, and the current
%   input vector, U.
%
%   FLAG   RESULT             DESCRIPTION
%   -----  ------             -------------------------------------------
%   0      [SIZES,X0,STR,TS]  Initialization, return system sizes in SYS,
%                             initial state in X0, state ordering strings
%                             in STR, and sample times in TS.
%   1      DX                 Return continuous state derivatives in SYS.
%   2      DS                 Update discrete states SYS = X(n+1)
%   3      Y                  Return outputs in SYS.
%   4      TNEXT              Return next time hit for variable step sample
%                             time in SYS.
%   5                         Reserved for future (root finding).
%   9      []                 Termination, perform any cleanup SYS=[].
%
%
%   The state vectors, X and X0 consists of continuous states followed
%   by discrete states.
%
%   Optional parameters, P1,...,Pn can be provided to the S-function and
%   used during any FLAG operation.
%
%   When SFUNC is called with FLAG = 0, the following information
%   should be returned:
%
%      SYS(1) = Number of continuous states.
%      SYS(2) = Number of discrete states.
%      SYS(3) = Number of outputs.
%      SYS(4) = Number of inputs.
%               Any of the first four elements in SYS can be specified
%               as -1 indicating that they are dynamically sized. The
%               actual length for all other flags will be equal to the
%               length of the input, U.
%      SYS(5) = Reserved for root finding. Must be zero.
%      SYS(6) = Direct feedthrough flag (1=yes, 0=no). The s-function
%               has direct feedthrough if U is used during the FLAG=3
%               call. Setting this to 0 is akin to making a promise that
%               U will not be used during FLAG=3. If you break the promise
%               then unpredictable results will occur.
%      SYS(7) = Number of sample times. This is the number of rows in TS.
%
%
%      X0     = Initial state conditions or [] if no states.
%
%      STR    = State ordering strings which is generally specified as [].
%
%      TS     = An m-by-2 matrix containing the sample time
%               (period, offset) information. Where m = number of sample
%               times. The ordering of the sample times must be:
%
%               TS = [0      0,      : Continuous sample time.
%                     0      1,      : Continuous, but fixed in minor step
%                                      sample time.
%                     PERIOD OFFSET, : Discrete sample time where
%                                      PERIOD > 0 & OFFSET < PERIOD.
%                     -2     0];     : Variable step discrete sample time
%                                      where FLAG=4 is used to get time of
%                                      next hit.
%
%               There can be more than one sample time providing
%               they are ordered such that they are monotonically
%               increasing. Only the needed sample times should be
%               specified in TS. When specifying than one
%               sample time, you must check for sample hits explicitly by
%               seeing if
%                  abs(round((T-OFFSET)/PERIOD) - (T-OFFSET)/PERIOD)
%               is within a specified tolerance, generally 1e-8. This
%               tolerance is dependent upon your model's sampling times
%               and simulation time.
%
%               You can also specify that the sample time of the S-function
%               is inherited from the driving block. For functions which
%               change during minor steps, this is done by
%               specifying SYS(7) = 1 and TS = [-1 0]. For functions which
%               are held during minor steps, this is done by specifying
%               SYS(7) = 1 and TS = [-1 1].

%   Copyright 1990-2002 The MathWorks, Inc.
%   $Revision: 1.18 $

%
% The following outlines the general structure of an S-function.
%
global S NOM


S= eye(NOM,NOM);

switch flag,

  %%%%%%%%%%%%%%%%%%
  % Initialization %
  %%%%%%%%%%%%%%%%%%
  case 0,
    [sys,x0,str,ts]=mdlInitializeSizes(X0);

  %%%%%%%%%%
  % Update %
  %%%%%%%%%%
  case 2,
   sys=mdlUpdate(t,x,u,H,G);

  %%%%%%%%%%%
  % Outputs %
  %%%%%%%%%%%
  case 3,
    sys=mdlOutputs(t,x);
    
  case {1, 4, 9}
    sys=[];
  %%%%%%%%%%%%%%%%%%%%
  % Unexpected flags %
  %%%%%%%%%%%%%%%%%%%%
  otherwise
    error(['Unhandled flag = ',num2str(flag)]);

end

% end sfuntmpl

%
%=============================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=============================================================================
%
function [sys,x0,str,ts]=mdlInitializeSizes(X0)
global P Q R NOM NOS


% call simsizes for a sizes structure, fill it in and convert it to a
% sizes array.
%
% Note that in this example, the values are hard coded.  This is not a
% recommended practice as the characteristics of the block are typically
% defined by the S-function parameters.
%
%j1 = 1;
sizes = simsizes;

sizes.NumContStates  = 0;
sizes.NumDiscStates  = 7;
sizes.NumOutputs     = 7;
sizes.NumInputs      = NOM+1;
sizes.DirFeedthrough = 0;
sizes.NumSampleTimes = 1;   % at least one sample time is needed

sys = simsizes(sizes);

%
% initialize the initial conditions
%
x0  = X0;

%
% str is always an empty matrix
%
str = [];

%
% initialize the array of sample times
%
ts  = [-1 0];

% end mdlInitializeSizes

%=============================================================================
% mdlUpdate
% Handle discrete state updates, sample time hits, and major time step
% requirements.
%=============================================================================
%
function sys=mdlUpdate(t,x,u,H,G)

global P Q R S NOM NOS T 

z = u(1:4);
du= u(5);

A= [-0.033   0.0001  0.0  -9.81 du  0   0  
    0.1680  -0.387   260   0.0  0   du  0    
    0.005   -0.0064 -0.55  0.0  0   0   du 
    0.0      0.0     1.0   0.0  0   0   0
     0       0       0     0    1   0   0
     0       0       0     0    0   1   0     
     0       0       0     0    0   0   1];
     
              
       
 H = [1 0 0 0 0 0 0
      0 1 0 0 0 0 0
      0 0 1 0 0 0 0
      0 0 0 1 0 0 0];    

delx = (1.0d-7)*x;
An = stlin(x,A,delx);
x = rk4int(x,A,T);
F  = expm(An*T);
P = F*P*F' + G*Q*G';
S = H*P*H' + R;	
K = P*H'*inv(S);
y = H*x ;
Nu= z - y;
x = (x + K*Nu)';
P =(eye(NOS)-K*H)*P;
sys = x;

function  x = rk4int(x,F,T)

nx = length(x);
xp=state(F,x);
dt=0.5*T;

for i=1:nx
rk1(i)=dt*xp(i);
x1(i)=x(i)+rk1(i);
end

xp=state(F, x1');

for i=1:nx
rk2(i)=dt*xp(i);
x1(i)=x(i)+rk2(i);
end

xp=state(F, x1');

for i=1:nx
rk3(i)=dt*xp(i);
x1(i)=x(i)+2*rk3(i);
end

xp=state(F, x1');

for i=1:nx
rk4(i)=dt*xp(i);
x(i)=x(i)+(rk1(i)+2.0*(rk2(i)+rk3(i))+rk4(i))/3;
end

% stlin.m//linearize state equations using finite differences
function [a] = stlin(xin,A,delx)
nx = length(xin);
xem=xin;
xp1 = state(A,xin);
for i=1:nx 
         if delx(i)== 0;
            del2=1.0d-5;
         else
            del2=delx(i);
         end
   xin(i)=xin(i)+del2;
   xp2=state(A,xin);
       for j=1:nx
           a(j,i)=(xp2(j)-xp1(j))/del2;
       end                                   
xin=xem;
end

function xp=state(F,x)
xp = F*x;
xp(5:7) = 0;

% end mdlUpdate

%==========================================================================
%==
% mdlOutputs
% Return the block outputs.
%=============================================================================
%
function sys=mdlOutputs(t,x)

sys = x(1:7);

% end mdlOutputs

